@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2010, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2011, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -65,8 +65,6 @@
 #include <curl/curl.h>
 #include "urldata.h"
 #include "sendf.h"
-#include "easyif.h" /* for Curl_convert_... prototypes */
-
 #include "if2ip.h"
 #include "hostip.h"
 #include "progress.h"
@@ -86,6 +84,7 @@
 #include "url.h"
 #include "rawstr.h"
 #include "strtoofft.h"
+#include "http_proxy.h"
 
 #define _MPRINTF_REPLACE /* use our functions only */
 #include <curl/mprintf.h>
@@ -128,7 +127,8 @@ const struct Curl_handler Curl_handler_pop3 = {
   ZERO_NULL,                        /* perform_getsock */
   pop3_disconnect,                  /* disconnect */
   PORT_POP3,                        /* defport */
-  PROT_POP3                         /* protocol */
+  CURLPROTO_POP3,                   /* protocol */
+  PROTOPT_CLOSEACTION               /* flags */
 };
 
 
@@ -151,7 +151,8 @@ const struct Curl_handler Curl_handler_pop3s = {
   ZERO_NULL,                        /* perform_getsock */
   pop3_disconnect,                  /* disconnect */
   PORT_POP3S,                       /* defport */
-  PROT_POP3 | PROT_POP3S | PROT_SSL  /* protocol */
+  CURLPROTO_POP3 | CURLPROTO_POP3S, /* protocol */
+  PROTOPT_CLOSEACTION | PROTOPT_SSL /* flags */
 };
 #endif
 
@@ -174,7 +175,8 @@ static const struct Curl_handler Curl_handler_pop3_proxy = {
   ZERO_NULL,                            /* perform_getsock */
   ZERO_NULL,                            /* disconnect */
   PORT_POP3,                            /* defport */
-  PROT_HTTP                             /* protocol */
+  CURLPROTO_HTTP,                       /* protocol */
+  PROTOPT_NONE                          /* flags */
 };
 
 
@@ -197,7 +199,8 @@ static const struct Curl_handler Curl_handler_pop3s_proxy = {
   ZERO_NULL,                            /* perform_getsock */
   ZERO_NULL,                            /* disconnect */
   PORT_POP3S,                           /* defport */
-  PROT_HTTP                             /* protocol */
+  CURLPROTO_HTTP,                       /* protocol */
+  PROTOPT_NONE                          /* flags */
 };
 #endif
 #endif
@@ -233,6 +236,7 @@ static void state(struct connectdata *conn,
     "PASS",
     "STARTTLS",
     "LIST",
+    "LIST_SINGLE",
     "RETR",
     "QUIT",
     /* LAST */
@@ -271,6 +275,15 @@ static int pop3_getsock(struct connectdata *conn,
   return Curl_pp_getsock(&conn->proto.pop3c.pp, socks, numsocks);
 }
 
+#ifdef USE_SSL
+static void pop3_to_pop3s(struct connectdata *conn)
+{
+  conn->handler = &Curl_handler_pop3s;
+}
+#else
+#define pop3_to_pop3s(x)
+#endif
+
 /* for STARTTLS responses */
 static CURLcode pop3_state_starttls_resp(struct connectdata *conn,
                                          int pop3code,
@@ -283,16 +296,19 @@ static CURLcode pop3_state_starttls_resp(struct connectdata *conn,
   if(pop3code != 'O') {
     failf(data, "STARTTLS denied. %c", pop3code);
     result = CURLE_LOGIN_DENIED;
+    state(conn, POP3_STOP);
   }
   else {
     /* Curl_ssl_connect is BLOCKING */
     result = Curl_ssl_connect(conn, FIRSTSOCKET);
     if(CURLE_OK == result) {
-      conn->protocol |= PROT_POP3S;
+      pop3_to_pop3s(conn);
       result = pop3_state_user(conn);
     }
+    else {
+      state(conn, POP3_STOP);
+    }
   }
-  state(conn, POP3_STOP);
   return result;
 }
 
@@ -423,6 +439,24 @@ static CURLcode pop3_state_list_resp(struct connectdata *conn,
   return result;
 }
 
+/* for LIST response with a given message */
+static CURLcode pop3_state_list_single_resp(struct connectdata *conn,
+                                     int pop3code,
+                                     pop3state instate)
+{
+  CURLcode result = CURLE_OK;
+  struct SessionHandle *data = conn->data;
+  (void)instate; /* no use for this yet */
+
+  if(pop3code != 'O') {
+    failf(data, "Invalid message. %c", pop3code);
+    result = CURLE_REMOTE_FILE_NOT_FOUND;
+  }
+
+  state(conn, POP3_STOP);
+  return result;
+}
+
 /* start the DO phase for RETR */
 static CURLcode pop3_retr(struct connectdata *conn)
 {
@@ -447,7 +481,10 @@ static CURLcode pop3_list(struct connectdata *conn)
   if(result)
     return result;
 
-  state(conn, POP3_LIST);
+  if (strlen(pop3c->mailbox))
+    state(conn, POP3_LIST_SINGLE);
+  else
+    state(conn, POP3_LIST);
   return result;
 }
 
@@ -481,7 +518,7 @@ static CURLcode pop3_statemach_act(struct connectdata *conn)
       if(data->set.ftp_ssl && !conn->ssl[FIRSTSOCKET].use) {
         /* We don't have a SSL/TLS connection yet, but SSL is requested. Switch
            to TLS connection now */
-        result = Curl_pp_sendf(&pop3c->pp, "STARTTLS", NULL);
+        result = Curl_pp_sendf(&pop3c->pp, "STLS");
         state(conn, POP3_STARTTLS);
       }
       else
@@ -510,6 +547,10 @@ static CURLcode pop3_statemach_act(struct connectdata *conn)
       result = pop3_state_list_resp(conn, pop3code, pop3c->state);
       break;
 
+    case POP3_LIST_SINGLE:
+      result = pop3_state_list_single_resp(conn, pop3code, pop3c->state);
+      break;
+
     case POP3_QUIT:
       /* fallthrough, just stop! */
     default:
@@ -600,7 +641,7 @@ static CURLcode pop3_connect(struct connectdata *conn,
   if(CURLE_OK != result)
     return result;
 
-  /* We always support persistant connections on pop3 */
+  /* We always support persistent connections on pop3 */
   conn->bits.close = FALSE;
 
   pp->response_time = RESP_TIMEOUT; /* set default response time-out */
@@ -608,7 +649,6 @@ static CURLcode pop3_connect(struct connectdata *conn,
   pp->endofresp = pop3_endofresp;
   pp->conn = conn;
 
-#if !defined(CURL_DISABLE_HTTP) && !defined(CURL_DISABLE_PROXY)
   if(conn->bits.tunnel_proxy && conn->bits.httpproxy) {
     /* for POP3 over HTTP proxy */
     struct HTTP http_proxy;
@@ -635,9 +675,8 @@ static CURLcode pop3_connect(struct connectdata *conn,
     if(CURLE_OK != result)
       return result;
   }
-#endif /* !CURL_DISABLE_HTTP && !CURL_DISABLE_PROXY */
 
-  if(conn->protocol & PROT_POP3S) {
+  if(conn->handler->protocol & CURLPROTO_POP3S) {
     /* BLOCKING */
     /* POP3S is simply pop3 with SSL for the control channel */
     /* now, perform the SSL initialization for this socket */
@@ -677,6 +716,7 @@ static CURLcode pop3_done(struct connectdata *conn, CURLcode status,
 {
   struct SessionHandle *data = conn->data;
   struct FTP *pop3 = data->state.proto.pop3;
+  struct pop3_conn *pop3c = &conn->proto.pop3c;
   CURLcode result=CURLE_OK;
   (void)premature;
 
@@ -693,6 +733,9 @@ static CURLcode pop3_done(struct connectdata *conn, CURLcode status,
     result = status;      /* use the already set error code */
   }
 
+  Curl_safefree(pop3c->mailbox);
+  pop3c->mailbox = NULL;
+
   /* clear these for next connection */
   pop3->transfer = FTPTRANSFER_BODY;
 
@@ -730,7 +773,7 @@ CURLcode pop3_perform(struct connectdata *conn,
   /* If mailbox is empty, then assume user wants listing for mail IDs,
    * otherwise, attempt to retrieve the mail-id stored in mailbox
    */
-  if (strlen(pop3c->mailbox))
+  if (strlen(pop3c->mailbox) && !conn->data->set.ftp_list_only)
     result = pop3_retr(conn);
   else
     result = pop3_list(conn);
@@ -864,15 +907,12 @@ static CURLcode pop3_dophase_done(struct connectdata *conn,
                                   bool connected)
 {
   struct FTP *pop3 = conn->data->state.proto.pop3;
-  struct pop3_conn *pop3c = &conn->proto.pop3c;
   (void)connected;
 
   if(pop3->transfer != FTPTRANSFER_BODY)
     /* no data to transfer */
     Curl_setup_transfer(conn, -1, -1, FALSE, NULL, -1, NULL);
 
-  free(pop3c->mailbox);
-
   return CURLE_OK;
 }
 
