@@ -1,13 +1,24 @@
-/*****************************************************************************
+/***************************************************************************
  *                                  _   _ ____  _
  *  Project                     ___| | | |  _ \| |
  *                             / __| | | | |_) | |
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * $Id: lib504.c,v 1.28 2008-09-20 04:26:57 yangtse Exp $
- */
-
+ * Copyright (C) 1998 - 2011, Daniel Stenberg, <daniel@haxx.se>, et al.
+ *
+ * This software is licensed as described in the file COPYING, which
+ * you should have received as part of this distribution. The terms
+ * are also available at http://curl.haxx.se/docs/copyright.html.
+ *
+ * You may opt to use, copy, modify, merge, publish, distribute and/or sell
+ * copies of the Software, and permit persons to whom the Software is
+ * furnished to do so, under the terms of the COPYING file.
+ *
+ * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
+ * KIND, either express or implied.
+ *
+ ***************************************************************************/
 #include "test.h"
 
 #include <sys/types.h>
@@ -28,10 +39,10 @@
 int test(char *URL)
 {
   CURL *c;
-  int ret=0;
-  CURLM *m;
+  int res = 0;
+  CURLM *m = NULL;
   fd_set rd, wr, exc;
-  CURLMcode res;
+  CURLMcode ret;
   char done = FALSE;
   int running;
   int max_fd;
@@ -52,11 +63,12 @@ int test(char *URL)
     return TEST_ERR_MAJOR_BAD;
   }
 
-  /* the point here being that there must not run anything on the given
+  /* The point here is that there must not be anything running on the given
      proxy port */
-  curl_easy_setopt(c, CURLOPT_PROXY, libtest_arg2);
-  curl_easy_setopt(c, CURLOPT_URL, URL);
-  curl_easy_setopt(c, CURLOPT_VERBOSE, 1L);
+  if (libtest_arg2)
+    test_setopt(c, CURLOPT_PROXY, libtest_arg2);
+  test_setopt(c, CURLOPT_URL, URL);
+  test_setopt(c, CURLOPT_VERBOSE, 1L);
 
   if ((m = curl_multi_init()) == NULL) {
     fprintf(stderr, "curl_multi_init() failed\n");
@@ -65,9 +77,9 @@ int test(char *URL)
     return TEST_ERR_MAJOR_BAD;
   }
 
-  if ((res = curl_multi_add_handle(m, c)) != CURLM_OK) {
+  if ((ret = curl_multi_add_handle(m, c)) != CURLM_OK) {
     fprintf(stderr, "curl_multi_add_handle() failed, "
-            "with code %d\n", res);
+            "with code %d\n", ret);
     curl_multi_cleanup(m);
     curl_easy_cleanup(c);
     curl_global_cleanup();
@@ -93,10 +105,10 @@ int test(char *URL)
 
     fprintf(stderr, "curl_multi_perform()\n");
 
-    res = CURLM_CALL_MULTI_PERFORM;
+    ret = CURLM_CALL_MULTI_PERFORM;
 
-    while (res == CURLM_CALL_MULTI_PERFORM) {
-      res = curl_multi_perform(m, &running);
+    while (ret == CURLM_CALL_MULTI_PERFORM) {
+      ret = curl_multi_perform(m, &running);
       if (tutil_tvdiff(tutil_tvnow(), mp_start) >
           MULTI_PERFORM_HANG_TIMEOUT) {
         mp_timedout = TRUE;
@@ -112,15 +124,15 @@ int test(char *URL)
       CURLMsg *msg = curl_multi_info_read(m, &numleft);
       fprintf(stderr, "Expected: not running\n");
       if(msg && !numleft)
-        ret = 100; /* this is where we should be */
+        res = 100; /* this is where we should be */
       else
-        ret = 99; /* not correct */
+        res = 99; /* not correct */
       break;
     }
-    fprintf(stderr, "running == %d, res == %d\n", running, res);
+    fprintf(stderr, "running == %d, ret == %d\n", running, ret);
 
-    if (res != CURLM_OK) {
-      ret = 2;
+    if (ret != CURLM_OK) {
+      res = 2;
       break;
     }
 
@@ -132,7 +144,7 @@ int test(char *URL)
     fprintf(stderr, "curl_multi_fdset()\n");
     if (curl_multi_fdset(m, &rd, &wr, &exc, &max_fd) != CURLM_OK) {
       fprintf(stderr, "unexpected failured of fdset.\n");
-      ret = 3;
+      res = 3;
       break;
     }
     rc = select_test(max_fd+1, &rd, &wr, &exc, &interval);
@@ -144,14 +156,18 @@ int test(char *URL)
     if (mp_timedout) fprintf(stderr, "mp_timedout\n");
     fprintf(stderr, "ABORTING TEST, since it seems "
             "that it would have run forever.\n");
-    ret = TEST_ERR_RUNS_FOREVER;
+    res = TEST_ERR_RUNS_FOREVER;
   }
 
-  curl_multi_remove_handle(m, c);
+test_cleanup:
+
+  if(m) {
+    curl_multi_remove_handle(m, c);
+    curl_multi_cleanup(m);
+  }
   curl_easy_cleanup(c);
-  curl_multi_cleanup(m);
   curl_global_cleanup();
 
-  return ret;
+  return res;
 }
 
